#!/usr/bin/env python

#
# Copyright (c) 2011, MegaVersiya, LTD. All rights reserved.
#
# Redistribution and use in source and binary forms, with or
# without modification, are permitted provided that
# the following conditions are met:
# Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
# Neither the name of the MegaVersiya, LTD nor the names of its contributors
# may be used to endorse or promote products derived from this software
# without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
# OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT * LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
# OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
#

"""
Script checks installation id.

Usage example:
./check_installation.py -a http://192.168.100.26/main.svc?wsdl -t10 \
 -f /var/lib/netpolice/installation_id
"""

from __future__ import with_statement
import sys
import optparse

import suds


class UtilException(Exception):
    """Exception class for util functions."""

    def __init__(self, message, previous=None):
        Exception.__init__(self, message, previous)

        self.message = message
        self.previous = previous

    def __unicode__(self):
        return self.message


def get_cmd_options():
    """Return a dictionary of command line options."""
    parser = optparse.OptionParser()

    parser.add_option('-a', '--service-address', dest='service_address',
                   help='License service wsdl url.')

    parser.add_option('-t', '--timeout', dest='service_timeout',
                   help='License service connection timeout.')

    parser.add_option('-f', '--installation-file', dest='installation_file',
                   help='installation file')

    (options, _) = parser.parse_args()

    return options.__dict__


def build_options():
    """Return a dictionary of all options."""
    options = get_cmd_options()

    help_msg = '%s. Use -h for help.'

    if options['service_address'] is None:
        raise UtilException(help_msg % 'Service address required')

    if options['installation_file'] is None:
        raise UtilException(help_msg % 'Installation file required.')

    if options['service_timeout'] is None:
        options['service_timeout'] = 10
    else:
        try:
            options['service_timeout'] = int(options['service_timeout'])
        except ValueError:
            raise UtilException('Timeout should be an integer.')

    return options


def get_installation_id(filepath):
    """Return installation id from file with given filepath."""
    try:
        with (open(filepath)) as fp:
            return fp.readline().strip()
    except (OSError, IOError), exc:
        raise UtilException(
            'Installation file reading error: %s.' % unicode(exc)
        )


CONFIGURATION_ERROR = 101

SERVICE_CALL_ERROR = 301

CALL_SUCCESS = 0


def main():
    """Entry point."""

    try:
        options = build_options()

        installation_id = get_installation_id(
            options['installation_file']
        )
    except UtilException, exc:
        sys.stderr.write('%s\n' % unicode(exc))
        sys.exit(CONFIGURATION_ERROR)

    try:
        client = suds.client.Client(options['service_address'])
        client.set_options(timeout=options['service_timeout'])

        exec_result = client.service.CheckInstallationKey(installation_id)

        result = exec_result.CheckInstallationKeyResult

        if result.Code == CALL_SUCCESS:
            sys.exit(exec_result.isValid + 200)
        else:
            sys.exit(result.Code + 400)
    except Exception, exc:
        sys.stderr.write('%s\n' % unicode(exc))
        sys.exit(SERVICE_CALL_ERROR)


if __name__ == '__main__':
    main()
